/*****************************************************
* wmm2.c - Window manager manager 2 (using GTK)
* (c) 1997 Tomi Suviola
* date: 19.08.97
****************************************************/

#include  <gtk/gtk.h> 
#include  <stdio.h> 
#include  <unistd.h> 

#define MAXB 20 /* max buttons */

/* Global variables */

void quit();
void bprint();
char komento[MAXB][256];
GtkWidget *toplevel, *container;
int nbuttons;
int debug = 0;

int com[MAXB]={0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19};

/***********************************************************/
/* find resource file                                      */
/***********************************************************/
char *LookUpFiles( char *path, char *filename, int mode )
{
        char *find;

        if( !access( filename, mode ) ){
                find = (char *)calloc( strlen( filename )+1, 1 );
                strcpy( find, filename );
                return find;
        }
        if( path==NULL )        return NULL;
        do{
                find = (char *)calloc( strlen(path)+strlen(filename)+2, 1 );
                if( strchr( path, ':' )==NULL ){
                        strcpy( find, path );
                        sprintf( find+strlen(path), "/%s", filename );
                        path += strlen( path );
                }
                else{
                        strcpy( find, path );
                        sprintf( strchr( find, ':' ), "/%s", filename );
                        path = strchr( path, ':') + 1;

                }
                if( !access(find, mode ) ){
                        return find;
                }
                free( find );
        }
        while( *path!='\0' && *path!='\n' );
        return NULL;
}


/***********************************************************/
/* skip space in string                                    */
/***********************************************************/
char *SkipSpace( char *str )
{
        for( ; (*str==' ' || *str=='\t'); str++ );
        return str;
}


/***********************************************************/
/* parse resource file                                     */
/***********************************************************/
void ReadConfigFile( char *configfile )
{
        int lp;
        FILE *fp;
        char str[256], *file, *cmp;
        char name[256],ex[256],cmd[256];
        int i;
	GtkWidget *button;

        if( (file = LookUpFiles( NULL, configfile, R_OK ))==NULL )
                file = LookUpFiles( (char *)getenv("HOME"), configfile, R_OK );

        if( file==NULL || (fp=fopen( file, "r" ))==NULL ){
            printf( "Can't open file (%s). \n", configfile );
            if( file )      free( file );
                return;
        }

        nbuttons = 0;

        while( fgets( str, 256, fp )!=NULL ){
                cmp = str;
                if( *cmp == '#' ) continue;
                cmp = SkipSpace( cmp );
                if( *cmp == '\n' ) continue;

/* read name */
                if (*cmp == '"') {
                  i=1;  
                  while (cmp[i]!='"'){
                    name[i-1]=cmp[i];
                    i++;
		    if (i>=256) { 
		      printf("Error reading %s.\n",file); 
		      exit (1); 
		    }
		  }
		  name[i-1]='\0';
		  cmp=cmp+i+1;
		  cmp = SkipSpace( cmp );

		} else {
		  sscanf(cmp,"%s",name);
		  cmp=cmp+strlen(name);
		  cmp = SkipSpace( cmp );
		}

/* read command */
		sscanf(cmp,"%s",ex);
		cmp=cmp+strlen(ex);
		cmp = SkipSpace( cmp );

/* read commandline */
		if (*cmp == '"') {
		  i=1;  
		  while (cmp[i]!='"'){
		    cmd[i-1]=cmp[i];
		    i++;
		    if (i>=256) { 
		      printf("Error reading %s.\n",file); 
		      exit (1); 
		    }
		  }
		  cmd[i-1]='\0';
		  cmp=cmp+i+1;
		  cmp = SkipSpace( cmp );

		} else {
		  sscanf(cmp,"%s",cmd);
		  cmp=cmp+strlen(cmd);
		  cmp = SkipSpace( cmp );
		}

		if (debug) {
		  printf("name=%s\n",name);
		  printf("ex=%s\n",ex);
		  printf("cmd=%s\n",cmd);
		}
		if (!strcmp(ex,"exec")&&!strcmp(ex,"EXEC")
		    &&!strcmp(ex,"Exec")){ 
		  printf("Invalid command %s\n",ex); 
		  exit(1);
		}

/* line parsed. Adding button */
		
		strcpy(komento[nbuttons],cmd); 
		button = gtk_button_new_with_label (name);
		gtk_signal_connect (GTK_OBJECT (button ), "clicked",
                             GTK_SIGNAL_FUNC (bprint), &com[nbuttons]);
		gtk_box_pack_start(GTK_BOX(container), button, TRUE, TRUE, 0);
		gtk_widget_show (button);

		nbuttons++;
                if (nbuttons >=MAXB) {
		  nbuttons=MAXB-1;
		  printf("Too many items!\n");
		}
        }
        fclose( fp );
        free( file );

        return;
}


/***********************************************************/
/* quit                                                    */
/***********************************************************/
void quit(GtkWidget *w,gpointer *data)
{
 gtk_main_quit ();

}

/***********************************************************/
/* execution                                               */
/***********************************************************/
void  bprint(GtkWidget *w,gpointer *data)
{
   int pidi;
   int client;
   client = *data;

   if (debug) {
       printf("button %d\n",client);
       printf("(%s)\n",komento[client]);
   }    
   
   if( access( komento[client], X_OK ) ) {
      printf ("Cannot exec %s\n",komento[client]);
      return;
   }

   pidi = fork();
   if (!pidi) {      
      if( execl(komento[client],NULL) ) 
      {
        printf ("invalid exec %s\n",komento[client]);
        exit(1) ;
      } 
   }
}

/***********************************************************/
/* main                                                    */
/***********************************************************/
int main(int argc,char **argv)
{
       char file[256];
       int i;
       GtkWidget *box;

       gtk_init (&argc, &argv);
       toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);

       gtk_signal_connect (GTK_OBJECT (toplevel), "destroy",
                             GTK_SIGNAL_FUNC (quit), NULL);
       gtk_window_set_title (GTK_WINDOW (toplevel), "Wmm2");
       gtk_container_border_width (GTK_CONTAINER (toplevel), 10);

       strcpy(file,".wmmrc");
       
       for (i=1;i<argc;i++) {
         if (!strcmp(argv[i],"-f")) {
	   if (i<argc-1) {
 	     strncpy(file,argv[i+1],255);
	     i++;
	   } 
	 }
         else if (!strcmp(argv[i],"-d")) {
	   debug=1;
	 }
	 else {
	   printf("Usage: wmm2 [-f filename] [toolkit options]\n");
	   exit(1);
	 }
       }

       container = gtk_vbox_new(FALSE, 0);       
       gtk_container_add (GTK_CONTAINER (toplevel), container);

       box=gtk_button_new_with_label ("Quit");

       gtk_signal_connect_object (GTK_OBJECT (box), "clicked",
                                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
                                    GTK_OBJECT (toplevel));

       gtk_box_pack_start(GTK_BOX(container), box, TRUE, TRUE, 0);

       gtk_widget_show(box);

       ReadConfigFile(file);

       gtk_widget_show (container);
       gtk_widget_show (toplevel);

       gtk_main ();
       return 0;
}
/***********************************************************/

